@@ -9,18 +9,21 @@ from aislab.gnrl.sf import *
 ####################################################################################
 # def trnd():
 ####################################################################################
-def permdl(x, T=7, Ne=None):
+def permdl(x, w=None, T=7, Ne=None):
     T = int(T)
     N = len(x)
     n = np.floor(N/T).astype(int)
     S = x[:n*T].reshape(n, T)
-    if N > n*T:
-        ss = np.empty((1, np.ceil(N/T).astype(int)*T - N))
-        ss[:] = np.nan
-        S = np.vstack((S, np.hstack((x[n*T:].T, ss))))
-    pm = np.nanmean(S, axis=0)
-    ind = T if len(x) % T == 0 else len(x) % T
-    pm = np.roll(pm, -ind)
+    if w is None:
+        if N > n*T:
+            ss = np.empty((1, np.ceil(N/T).astype(int)*T - N))
+            ss[:] = np.nan
+            S = np.vstack((S, np.hstack((x[n*T:].T, ss))))
+        pm = np.nanmean(S, axis=0)
+        ind = T if len(x) % T == 0 else len(x) % T
+        pm = np.roll(pm, -ind)
+    else:
+        pm=1
     return pm
 ####################################################################################
 # 	T determination = f(Rxx), f(fft), ...
