@@ -1,5 +1,8 @@
 import numpy as np
+import pandas as pd
 from aislab.gnrl.bf import *
+from aislab.gnrl.sf import *
+from aislab.gnrl.measr import *
 
 ##############################################################################
 ##############################################################################
@@ -10,9 +13,7 @@ def f_cs(X, args):
     from aislab.gnrl.measr import confm
     # Objective function for credit risk strategy optimization
     # needed for old GA
-    case = 1
     if isinstance(X, list):
-        case = -1
         X = np.array(X)
     # needed for old GA
 
@@ -31,24 +32,26 @@ def f_cs(X, args):
     lb2 = args['lb2']  # lower bound for cut-offs of the second scorecard
     ub1 = args['ub1']  # upper bound for cut-offs of the first scorecard
     ub2 = args['ub2']  # upper bound for cut-offs of the second scorecard
-    minx = args['minx']  # upper bound for cut-offs of the first scorecard
-    maxx = args['maxx']  # upper bound for cut-offs of the second scorecard
     if X.ndim == 1: X = np.array([X])
     N = len(X)
     F = np.full((N,), np.nan)
     i = 0
     for x in X:
-        x1 = x[:nc1, ]
-        x2 = x[nc1:, ]
-        # x1, ss = sort(c_(x[:nc1, ]))
-        # x2, ss = sort(c_(x[nc1:, ]))
-        # minx = np.min([minx, np.min(x)])  # upper bound for cut-offs of the first scorecard
-        # maxx = np.max([maxx, np.max(x)])  # upper bound for cut-offs of the second scorecard
-        # x1 = (x1 - minx) / (maxx - minx) * (ub1 - lb1) + lb1
-        # x2 = (x2 - minx) / (maxx - minx) * (ub2 - lb2) + lb2
-        # x1 = np.round(x1.flatten())
-        # x2 = np.round(x2.flatten())
-        if len(set(x1)) != len(x1) or len(set(x2)) != len(x2): return case*np.inf  # if there are 2 or more the same GB cut-offs: F = inf
+        # !!!!!
+        x1, ss = sort(c_(x[:nc1, ]))
+        x2, ss = sort(c_(x[nc1:, ]))
+        minx = np.min([args['minx'], np.min(x)])  # upper bound for cut-offs of the first scorecard
+        maxx = np.max([args['maxx'], np.max(x)])  # upper bound for cut-offs of the second scorecard
+        x1 = (x1 - minx) / (maxx - minx) * (ub1 - lb1) + lb1
+        x2 = (x2 - minx) / (maxx - minx) * (ub2 - lb2) + lb2
+        x1 = np.round(x1.flatten())
+        x2 = np.round(x2.flatten())
+        # !!!!!
+
+        if len(set(x1)) != len(x1) or len(set(x2)) != len(x2):
+            F[i] = np.inf
+            i += 1
+            continue # if there are 2 or more the same GB cut-offs: F = inf
         try:
             GB_zones = cut(data['Score_GB'].values, x1).flatten()
             GBR_zones = cut(data['Score_GBR'].values, x2).flatten()
@@ -96,17 +99,313 @@ def f_cs(X, args):
         F[i] = w[0] * F1 + w[1] * F2 + w[2] * F3 + w[3] * F4 + w[4] * F5 + w[5] * F6 + w[6] * F7
 
         if np.isnan(F[i]):
-            F[i] = case * np.inf
-            return F
+            F[i] = np.inf
+            if i == len(F):
+                return F
         i += 1
-        # needed for old GA
-        if len(F) == 1: F = -F[0]
-        if isinstance(F, float) and np.isinf(F): F = -np.inf
-        # needed for old GA
+    #     # needed for old GA
+    #     if len(F) == 1: F = -F[0]
+    #     if isinstance(F, float) and np.isinf(F): F = -np.inf
+    # if len(X) > 1: F = -F
+    #     # needed for old GA
+#     if isinstance(X, list): F = F[0]
+
     return F
 ##############################################################################
+def bprob(Strat, data, w, args_new, args_old):
+    if Strat == 'NewClients_Product1':
+        data = data[['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB_N1', 'Score_GBR_N1', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == True]
+        data = data[data['ProductType'] == 1]
+        args = args_new
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    elif Strat == 'NewClients_Product2':
+        data = data[['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB_N2', 'Score_GBR_N2', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == True]
+        data = data[data['ProductType'] == 2]
+        args = args_new
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    elif Strat == 'OldClients_Product1':
+        data = data[['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB_O1', 'Score_GBR_O1', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == False]
+        data = data[data['ProductType'] == 1]
+        data['Bad'][data['Reject'] == 1] = 0
+        args = args_old
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    elif Strat == 'OldClients_Product2':
+        data = data[['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB_O2', 'Score_GBR_O2', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'ProductType', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == False]
+        data = data[data['ProductType'] == 2]
+        data['Bad'][data['Reject'] == 1] = 0
+        args = args_old
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    elif Strat == 'NewClients':
+        data = data[['APP_AccountID', 'NewClient', 'Score_GB_New', 'Score_GBR_New', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == True]
+        args = args_new
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    elif Strat == 'OldClients':
+        data = data[['APP_AccountID', 'NewClient', 'Score_GB_Old', 'Score_GBR_Old', 'Good', 'Bad', 'Reject']]
+        data.columns = ['APP_AccountID', 'NewClient', 'Score_GB', 'Score_GBR', 'Good', 'Bad', 'Reject']
+        data = data[data['NewClient'] == False]
+        data['Bad'][data['Reject'] == 1] = 0
+        args = args_old
+        args['lb1'] = np.min(data['Score_GB'])
+        args['ub1'] = np.max(data['Score_GB'])
+        args['lb2'] = np.min(data['Score_GBR'])
+        args['ub2'] = np.max(data['Score_GBR'])
+        args['minx'] = np.min(np.min(data[['Score_GB', 'Score_GBR']]))
+        args['maxx'] = np.max(np.max(data[['Score_GB', 'Score_GBR']]))
+    args['w'] = w
+    args['data'] = data
+    return data, args
+
+###############################################################################
+def tables(x, args):
+    from aislab.gnrl.sf import sort
+    from aislab.dp_feng.binenc import cut
+
+    data = args['data']  # data set
+    nc1 = args['nc1']  # number of cut-offs w.r.t. first (GB) scorecard
+    nc2 = args['nc2']  # number of cut-offs w.r.t. second (GBR) scorecard
+
+    lb1 = args['lb1']  # lower bound for cut-offs of the first scorecard
+    lb2 = args['lb2']  # lower bound for cut-offs of the second scorecard
+    ub1 = args['ub1']  # upper bound for cut-offs of the first scorecard
+    ub2 = args['ub2']  # upper bound for cut-offs of the second scorecard
+    minx = np.min([args['minx'], np.min(x)])  # upper bound for cut-offs of the first scorecard
+    maxx = np.max([args['maxx'], np.max(x)])  # upper bound for cut-offs of the second scorecard
+
+
+    x1, ss = sort(c_(x[:nc1, ]))
+    x2, ss = sort(c_(x[nc1:, ]))
+    minx = np.min([minx, np.min(x)])  # upper bound for cut-offs of the first scorecard
+    maxx = np.max([maxx, np.max(x)])  # upper bound for cut-offs of the second scorecard
+    x1 = (x1 - minx)/(maxx - minx)*(ub1 - lb1) + lb1
+    x2 = (x2 - minx)/(maxx - minx)*(ub2 - lb2) + lb2
+    x1 = np.round(x1.flatten())
+    x2 = np.round(x2.flatten())
+
+    cs = np.hstack((x1, x2))
+
+    GB_zones = cut(data['Score_GB'].values, x1).flatten()
+    GBR_zones = cut(data['Score_GBR'].values, x2).flatten()
+
+    names1 = ['GB_A']
+    for i in range(nc1 - 1): names1.append('GB_G' + str(i+1))
+    names1.append('GB_R')
+    names2 = ['GBR_A']
+    for i in range(nc2 - 1): names2.append('GBR_G' + str(i+1))
+    names2.append('GBR_R')
+    # confusion matrix
+    good = confm(-GB_zones, -GBR_zones, w=data['Good'].values, ux1=range(1-len(names1), 1), ux2=range(1-len(names2), 1)).astype(int)
+    bad = confm(-GB_zones, -GBR_zones, w=data['Bad'].values, ux1=range(1-len(names1), 1), ux2=range(1-len(names2), 1)).astype(int)
+    reject = confm(-GB_zones, -GBR_zones, w=data['Reject'].values, ux1=range(1-len(names1), 1), ux2=range(1-len(names2), 1)).astype(int)
+
+    # n1 = len(names1)
+    # for i in range(n1):
+    #     if i not in np.unique(GB_zones):
+    #         good = np.hstack((good[:, :i], nans((n1, 1)), good[:, (i+1):]))
+    # n2 = len(names2)
+    # for i in range(n2):
+    #     if i not in np.unique(GBR_zones):
+    #         good = np.vstack((good[:i, :], nans((1, n2)), good[(i+1):, :]))
+
+
+    good = pd.DataFrame(good, index=names1, columns=names2)
+    bad = pd.DataFrame(bad, index=names1, columns=names2)
+    reject = pd.DataFrame(reject, index=names1, columns=names2)
+
+    bad_rate = bad/(good + bad)*100
+    total_gb = good + bad
+    total = total_gb + reject
+    return good, bad, reject, total_gb, total, bad_rate, cs
+
 ##############################################################################
-###################################################################################
+def tables2(G, B, R, TGB, TGBR, BR, args):
+    # GB Scorecard
+    nc1 = args['nc1']  # number of cut-offs w.r.t. first (GB) scorecard
+    nc2 = args['nc2']  # number of cut-offs w.r.t. first (GB) scorecard
+    names1 = ['Accept']
+    for i in range(nc1 - 1): names1.append('GB_G' + str(i+1))
+    names1.append('Reject')
+    names1.append('Total')
+    names2 = ['Accept']
+    for i in range(nc2 - 1): names2.append('GB_G' + str(i+1))
+    names2.append('Reject')
+    names2.append('Total')
+    G1 = np.sum(G, axis=1)
+    B1 = np.sum(B, axis=1)
+    R1 = np.sum(R, axis=1)
+    BadRej = B1 + R1
+    TGBR1 = np.sum(TGBR, axis=1)
+    TotalPrc = TGBR1/np.sum(TGBR1)*100
+    BR = B1/(G1 + B1)*100
+    BadAndRejectRate = BadRej/(G1 + BadRej)*100
+    T1 = np.vstack((G1, B1, R1, BadRej, TGBR1, np.round(TotalPrc, 1), np.round(BR, 1), np.round(BadAndRejectRate, 1)))
+    BR1 = np.sum(B1)/np.sum(G1 + B1)
+    BadAndRejectRate1 = np.sum(B1 + R1)/np.sum(G1 + B1 + R1)
+    Tot1 = [np.sum(np.sum(G1)), np.sum(np.sum(B1)), np.sum(np.sum(R1)), np.sum(np.sum(BadRej)), np.sum(np.sum(TGBR1)),
+            np.sum(np.sum(TotalPrc)), BR1, BadAndRejectRate1]
+    Tot = np.round(np.array([Tot1]).T, 1)
+    T1 = np.hstack((T1, Tot))
+    T1 = pd.DataFrame(T1, index=['Good', 'Bad', 'Reject', 'BadAndReject', 'Total', 'TotalPrc', 'BadRate', 'BadAndRejectRate'], columns=names1)
+    # GBR Scorecard
+    G2 = np.sum(G, axis=0)
+    B2 = np.sum(B, axis=0)
+    R2 = np.sum(R, axis=0)
+    BadRej = B2 + R2
+    TGBR2 = np.sum(TGBR, axis=0)
+    TotalPrc = TGBR2/np.sum(TGBR2)*100
+    BR = B2/(G2 + B2)*100
+    BadAndRejectRate = BadRej/(G2 + BadRej)*100
+    T2 = np.vstack((G2, B2, R2, BadRej, TGBR2, np.round(TotalPrc, 1), np.round(BR, 1), np.round(BadAndRejectRate, 1)))
+    BR1 = np.sum(B2)/np.sum(G2 + B2)
+    BadAndRejectRate1 = np.sum(B2 + R2)/np.sum(G2 + B2 + R2)
+    Tot1 = [np.sum(np.sum(G2)), np.sum(np.sum(B2)), np.sum(np.sum(R2)), np.sum(np.sum(BadRej)), np.sum(np.sum(TGBR2)),
+            np.sum(np.sum(TotalPrc)), BR1, BadAndRejectRate1]
+    Tot = np.round(np.array([Tot1]).T, 2)
+    T2 = np.hstack((T2, Tot))
+    T2 = pd.DataFrame(T2, index=['Good', 'Bad', 'Reject', 'BadAndReject', 'Total', 'TotalPrc', 'BadRate', 'BadAndRejectRate'], columns=names2)
+    return T1, T2
+###############################################################################
+def visualz(cs, F_opt, args, s, w, Strat, mut_rate, mut_met, mut_stdev, stop_itrConstF):
+    G, B, R, TGB, TGBR, BR, cs = tables(cs, args)
+    T1, T2 = tables2(G, B, R, TGB, TGBR, BR, args)  # if ~np.isinf(G):...
+
+    print('============================================')
+    print(Strat, ' - Report')
+    print('============================================')
+    T = {'': ['constraint', 'weight'],
+         'BRA': [args['BRA'], w[0]],
+         'TA': [args['TA'], w[0]],
+         'BRR': [args['BRR'], w[1]],
+         'TR': [args['TR'], w[1]],
+         'mindBR': [args['mindBR'], w[2]],
+         'minNb': [args['minNb'], w[3]],
+         'minNgb': [args['minNgb'], w[4]],
+         'NaN': [None, w[5]],
+         'min_max_cell': [None, w[6]],
+         'mutatRate': [mut_rate, None],
+         'mutMethod': [mut_met, 'NaN'],
+         'mutStdev': [mut_stdev, None],
+         'stop_itrConstF': [stop_itrConstF, None]
+        }
+    print(pd.DataFrame(T).set_index(''))
+    print('objF: ', F_opt)
+
+    lus = int(np.max(np.max(s)))
+    GG = np.full((lus, 1), np.nan)
+    BB = np.full((lus, 1), np.nan)
+    RR = np.full((lus, 1), np.nan)
+    TT = np.full((lus, 1), np.nan)
+    BRi = np.full((lus, 1), np.nan)
+
+    for i in range(1, lus + 1):
+        ind = s == i
+        GG[i-1] = sum(G.values[ind])
+        BB[i-1] = sum(B.values[ind])
+        RR[i-1] = sum(R.values[ind])
+        TT[i-1] = sum(TGBR.values[ind])
+
+    BRi = BB/(GG + BB)*100
+    TT = TT/sum(TT)*100
+    CumGG1 = np.cumsum(GG)
+    CumBB1 = np.cumsum(BB)
+    CumGG2 = np.cumsum(GG[::-1])[::-1]
+    CumBB2 = np.cumsum(BB[::-1])[::-1]
+
+    CumTT1 = c_(np.cumsum(TT))
+    CumTT2 = c_(np.cumsum(TT[::-1])[::-1])
+    CumBR1 = c_(CumBB1/(CumGG1 + CumBB1))
+    CumBR2 = c_(CumBB2/(CumGG2 + CumBB2))
+
+    nn = range(int(np.min(s)), int(np.max(s) + 1))
+    print('\n')
+    #print(np.hstack((np.array([nn]).T, BRi, CumBR1, CumBR2, TT, CumTT)))
+    Stats = pd.DataFrame(np.hstack((np.round(BRi, 2), np.round(TT, 2), np.round(CumBR1*100, 2), np.round(CumTT1, 2), np.round(CumBR2*100, 2), np.round(CumTT2, 2))))/100
+    Stats = Stats.dropna()
+    Stats.columns = ['BadRate', 'TotalPrc', 'CumBR', 'CumTotPrc', 'RevCumBR', 'RevCumTotPrc']
+    print(Stats.to_string(index=False))
+    print('\nCut-offsGB: ', cs[:args['nc1']])
+    print('Cut-offsGBR: ', cs[args['nc1']:])
+
+
+    print('\nBadRate: ')
+    np.set_printoptions(suppress=True)
+    print(np.round(BR, 2))
+
+    print('\nTotal: ')
+    print(TGBR)
+    print('\nGood: ')
+    print(G)
+    print('\nBad: ')
+    print(B)
+    print('\nGood&Bad: ')
+    print(TGB)
+    print('\nReject: ')
+    print(R)
+
+    # names = {1: 'Accept', 2: 'LowRisk', 3: 'MidRisk', 4:'HighRisk', 5: 'Reject'}
+    # m, n = s.shape
+    # Segment = np.full((m*n, 1), 'xxxxxxxxxx')
+    # Scorecard_GB = np.full((m*n, 1), 'xxxxxxxxx')
+    # Scorecard_GBR = np.full((m*n, 1), 'xxxxxxxxx')
+    # Colour = np.full((m*n, 1), 'xxxxxxxxx')
+    # Code = nans((m*n, 1))
+    # segment = 'OldClients'; s = s_old; colour = {1: 'green', 2: 'green', 3: 'green', 4: 'green', 5:'green', 6:'green', 7:'gray', 8:'gray', 9:'gray', 10:'red', 11:'red'}
+    # segment = 'NewClients'; s = s_new; colour = {1: 'green', 2: 'green', 3: 'green', 4: 'green', 5:'green', 6:'green', 7:'green', 8:'gray', 9:'gray', 10:'red', 11:'red', 12:'red'}
+    #
+    # k = 0
+    # for i in range(m):      # SC_GB
+    #     for j in range(n):  # SC_GBR
+    #         Segment[k, 0] = segment
+    #         Scorecard_GB[k, 0] = names[i+1]
+    #         Scorecard_GBR[k, 0] = names[j+1]
+    #         Colour[k, 0] = colour[s[i, j]]
+    #         Code[k, 0] = s[i, j]
+    #         k += 1
+    #
+    # df = pd.DataFrame({    'Segment': Segment.flatten(),
+    #      'Scorecard_GB': Scorecard_GB.flatten(),
+    #     'Scorecard_GBR': Scorecard_GBR.flatten(),
+    #            'Colour': Colour.flatten(),
+    #              'Code': Code.flatten()})
+    # df.to_csv('/home/user/Desktop/Saso/AutoSCDev/data/sc_Stik_07/python/'+'RiskZones'+segment+'.csv')
+    return cs, G, B, R, TGB, TGBR, BR, T1, T2
+
+###############################################################################
+###############################################################################
 
 # exp1:                                                 # y = a + b*e^x
 # exp2:                                                 # y = a + b*e^(x + c)
