@@ -26,12 +26,12 @@ x = pd.read_csv(dpath + 'x_samples_250000.csv')
 y = pd.read_csv(dpath + 'y_samples_250000.csv')
 cnf = pd.read_csv(cpath)
 
-N = 1000 # x.shape[0] #
+N =  1000 # x.shape[0] #
 
 x = x.iloc[:N, :]
 y = y.iloc[:N, :].values
 
-tic2() # Overall time
+tic1() # Overall time
 
 cname = cnf['cnames'].tolist()
 xtp = cnf['xtp'].values
@@ -43,28 +43,27 @@ ytp = ['bin']
 y[find(np.isnan(y) == 1)] = 0
 w[find(np.isnan(w) == 1)] = 0
 
-dsp = 1
+dsp = False # 'all' #
 order = order.values
 dlm = '$'
 # 1. All categorical vars to int
 tic()
-xe = enc_int(x, cname, xtp, vtp, order, dsp, dlm)
-toc('INT-ENCODING')
+xe = enc_int(x, cname, xtp, vtp, order, dsp, dlm=dlm)
+toc('ENCODING to INTEGER')
 xe.to_csv('data_enc_int.csv')
 
 # 2. BINNING
 tic()
-ub = ubng(xe, xtp, w, y=y, ytp=ytp, cnames=cname, dsp=True)     # unsupervised binning
-toc('UBNG')
+ub = ubng(xe, xtp, w, y=y, ytp=ytp, cnames=cname, dsp=dsp)     # unsupervised binning
+toc('UBNG total:')
 tic()
-sb = sbng(ub, dsp=True)            # supervised binning
-toc('SBNG')
+sb = sbng(ub, dsp=dsp)            # supervised binning
+toc('SBNG total:')
 
 # 3. ONE HOT ENCODING
 tic()
-
 x_mdl, cnames_mdl = enc_apl(sb, xe, met_enc)
-toc('ENCODING')
+toc('APPLY ENCODING - ' + met_enc)
 
 dat = pd.DataFrame(np.hstack((x_mdl, y, w)))
 cnames_mdl = cnames_mdl
@@ -90,18 +89,18 @@ ivi = np.array([]).astype(int) # c_(np.arange(1, int(np.floor(x_dv.shape[1]/3)))
 afcnv = 1e-6
 
 tic()
-mdl_lin = swlinr(x_mdl, y, w, cnames=cnames_mdl, met=met, mtp=mtp, ivi=ivi)
+mdl_lin = swlinr(x_mdl, y, w, cnames=cnames_mdl, met=met, mtp=mtp, ivi=ivi, dsp=dsp)
 toc('SWLINR')
 
 # 5. SW LOGISTIC REGRESSION
 # mtp = 'init' # 'empty' # 'full' #
 # ivi = mdl_lin[-1]['ivi']
+dsp_op=0 # 2 #
 tic()
-
-mdl_log = swlogr(x_mdl, y, w, cnames=cnames_mdl, SLE=1, SLS=1, met=met, mtp=mtp, ivi=ivi, dsp_op=2)
+mdl_log = swlogr(x_mdl, y, w, cnames=cnames_mdl, SLE=1, SLS=1, met=met, mtp=mtp, ivi=ivi, dsp=dsp, dsp_op=dsp_op)
 toc('SWLOGR')
 
-toc2('OVERALL') # Overall time
+toc1('OVERALL') # Overall time
 
 N = y.shape[0]
 xx_lin = np.hstack((ones((N,1)), x_mdl[:, mdl_lin[-1]['ivi'][1:]-1]))
