@@ -23,7 +23,7 @@ w = c_(dat['w'].values)         # weight
 y = c_(dat['y'].values)         # dependent variable
 cnames = dat.columns.tolist()[:-2]  # variables names
 
-# met = 'SWR' # 'BR' # 'FR' #
+met = 'SWR' # 'BR' # 'FR' #
 mtp = 'empty' # 'full' # 'init' #
 ivi = np.arange(1,int(np.floor(x.shape[1] / 3))+1)  # initial ivi - Indexes of Variables In the model
 
